/*
 * xpkFEAL.library V1.02
 *
 * Copyright (C) 1992,93,94 by Christian von Roques
 *
 */

static char version[]="$VER: xpkFEAL 1.03 "__AMIGADATE__"\r\n";

#include "WordOps.h"
#include "FEAL.h"
#include "FEAL_asm.h"

#define ECB_MODE 0      /* deactivated */
/* ECB: C[i]=FEAL(T[i])  T[i]=IFEAL(C[i]) */

#define CBC1_MODE 1     /* default */
/* CBCj: C[i]=FEAL(T[i]^C[i-j]) T[i]=IFEAL(C[i])^C[i-j] */

/* ... more modes will follow! ... */

#include <proto/exec.h>
#include <exec/memory.h>

#define NO_SUB_PRAGMAS

#include <libraries/xpksub.h>

#define A3000 XPKMF_A3000SPEED


// next upto flags packmem unpackmem packspeed unpackspeed ratio reserved
XMINFO Mode[] = {
        Mode+1,  4,A3000,0,0,238,244,0,0,"fastest",
        Mode+2,  8,A3000,0,0,171,174,0,0,"fast",
        Mode+3, 16,A3000,0,0,109,109,0,0,"safe",
        Mode+4, 32,A3000,0,0, 63, 63,0,0,"safer",
        NULL,  100,A3000,0,0, 34, 34,0,0,"safest"
};


static struct XpkInfo FealInfo = {
        1,                      /* info version */
        1,                      /* lib  version */
        0,                      /* master vers  */
        0,                      /* ModesVersion */
        "FEAL",                 /* short name   */
        "Fast Encryption ALgorithm 1.03",
                                /* long name    */
        "FEAL-N with CBC1.  Password protects data with selectable safety.",
                                /* description  */
        'FEAL',                 /* 4 letter ID  */
        XPKIF_PK_CHUNK  |       /* flags        */
        XPKIF_UP_CHUNK  |
        XPKIF_MODES     |
        XPKIF_ENCRYPTION|
        XPKIF_NEEDPASSWD|
        XPKIF_MODES,
        0x40000000, /* 1GB */   /* max in chunk */
        0,                      /* min in chunk */
        32768,                  /* def in chunk */
        "Encrypting",           /* pk message   */
        "Decrypting",           /* up message   */
        "Encrypted",            /* pk past msg  */
        "Decrypted",            /* up past msg  */
        16,                     /* def mode     */
        0,                      /* pad          */
        &Mode[0]                /* modes        */
};


/*
 * Returns the info structure of our encryptor
 */
struct XpkInfo * __saveds __asm
XpksPackerInfo( void )
{
  return &FealInfo;
}


void __saveds __asm
XpksPackFree( REG __a0 XPARAMS* xpar )
{
  if (xpar->Sub[0]) {
    FreeMem((APTR)xpar->Sub[0], sizeof(tPartKeys));
  }
}


/*
 * This forces the next chunk to be decryptable independent from the
 * previous one. This is always the case in FEAL.
 */
long __saveds __asm
XpksPackReset( REG __a0 XPARAMS* xpar )
{
  return XPKERR_OK;
}


void __saveds __asm
XpksUnpackFree( REG __a0 XPARAMS* xpar )
{
  if (xpar->Sub[0]) {
    FreeMem((APTR)xpar->Sub[0], sizeof(tPartKeys));
  }
}


/*
 * Encrypt a chunk
 */
long __saveds __asm
XpksPackChunk( REG __a0 XPARAMS *xpar )
{
  Word64 Key = password(xpar->Password),
         T,LastC;
  ULONG  *r = (ULONG*)xpar->InBuf,
         *w = (ULONG*)xpar->OutBuf;
  LONG   len = xpar->InLen;
  int    N = xpar->Mode, i;
  UWORD  csum = 0;

  LastC.lft=LastC.rgt=0;

  N &= ~1;
  if (N<=0) N=2;
  if (N>MaxN) N=MaxN;

  if (xpar->OutBufLen < ((xpar->InLen+15) & ~7))
    return XPKERR_SMALLBUF;

  if (!xpar->Sub[0]) {
    if (!(xpar->Sub[0] = (LONG) AllocMem(sizeof(tPartKeys), MEMF_PUBLIC)))
      return XPKERR_NOMEM;
    MakePartKeys((Word16*)xpar->Sub[0], Key, N);
  }

  /* fill in FEAL-Headerinformation. */
  ((UBYTE*)xpar->OutBuf)[2] = CBC1_MODE;
  ((UBYTE*)xpar->OutBuf)[3] = N;
/*((UWORD*)xpar->OutBuf)[0] = 0; checksum, filled in later */
  w++;  /* skip FEAL-Header */

  while (len >= 8) {
    csum += *r;                                 /* update checksum      */
    T.lft = *r++ ^ LastC.lft;
    T.rgt = *r++ ^ LastC.rgt;
    FEAL_asm(&T, (Word64*)w, (Word16*)xpar->Sub[0], N);
    LastC.lft=*w++; LastC.rgt=*w++;
    len -= 8;
  }

  w[0]=r[0]; w[1]=r[1];				/* prepare the last block */
  for (i=len; i<7; ++i) ((UBYTE*)w)[i]=0;	/* with 0 to 7 data bytes */
  ((UBYTE*)w)[7]=len;				/* and len as last byte   */

  csum += *w;
  T.lft = *w++ ^ LastC.lft;
  T.rgt = *w++ ^ LastC.rgt;
  FEAL_asm(&T, (Word64*)(w-2), (Word16*)xpar->Sub[0], N);

  ((UWORD*)xpar->OutBuf)[0] = csum;             /* fill in checksum     */
  xpar->OutLen = (UBYTE*)w - (UBYTE*)xpar->OutBuf;

  return XPKERR_OK;
}


/*
 * Decrypt a chunk
 */
long __saveds __asm
XpksUnpackChunk( REG __a0 XPARAMS *xpar )
{
  Word64 Key = password(xpar->Password),
         LastC;
  ULONG  *r = (ULONG*)xpar->InBuf,
         *w = (ULONG*)xpar->OutBuf;
  UWORD  csum;
  LONG   len = xpar->InLen;
  int    N;

  LastC.lft=LastC.rgt=0;

  csum = ((UWORD*)xpar->InBuf)[0];              /* checksum             */
  N    = ((UBYTE*)xpar->InBuf)[3];              /* Number of rounds     */
  if ((((UBYTE*)xpar->InBuf)[2]!= CBC1_MODE) ||
      (N > MaxN)) {
    return XPKERR_VERSION;
  }
  r++; len-=4; /* skip FEAL-Header */

  if (xpar->OutBufLen) /* fixed xpkMaster with !=0 OutBufLen */
    if (xpar->OutBufLen < len-8) /* 8:Can't know #bytes in last Block */
      return XPKERR_SMALLBUF;

  if (!xpar->Sub[0]) {
    if (!(xpar->Sub[0] = (LONG)AllocMem(sizeof(tPartKeys), MEMF_PUBLIC)))
      return XPKERR_NOMEM;
    MakePartKeys((Word16*)xpar->Sub[0], Key, N);
    PermPartKeys((Word16*)xpar->Sub[0], N);
  }

  xpar->OutLen = len-8;	/* Bytes of last block will be added later */

  while (len > 0) {
    FEAL_asm((Word64*)r, (Word64*)w, (Word16*)xpar->Sub[0], N);
    csum -= *w++ ^= LastC.lft;
            *w++ ^= LastC.rgt;
    LastC.lft = *r++;
    LastC.rgt = *r++;
    len-=8;
  }

  xpar->OutLen += ((UBYTE*)w)[-1];

  if (csum == 0) {
    return XPKERR_OK;
  } else {
    return XPKERR_WRONGPW;
  }
}
